@@ -20,6 +20,7 @@
  */
 
 #include "FourSuite.h"
+
 #include <uriparser/Uri.h>
 
 
@@ -57,6 +58,7 @@ bool FourSuite::testAddOrRemoveBaseHelper(const char * ref, const char * base,
 	if (res != 0) {
 		uriFreeUriMembersA(&baseUri);
 		uriFreeUriMembersA(&relUri);
+		uriFreeUriMembersA(&expectedUri);
 		return false;
 	}
 
@@ -71,6 +73,8 @@ bool FourSuite::testAddOrRemoveBaseHelper(const char * ref, const char * base,
 	if (res != 0) {
 		uriFreeUriMembersA(&baseUri);
 		uriFreeUriMembersA(&relUri);
+		uriFreeUriMembersA(&expectedUri);
+		uriFreeUriMembersA(&transformedUri);
 		return false;
 	}
 
@@ -86,6 +90,7 @@ bool FourSuite::testAddOrRemoveBaseHelper(const char * ref, const char * base,
 	uriFreeUriMembersA(&baseUri);
 	uriFreeUriMembersA(&relUri);
 	uriFreeUriMembersA(&expectedUri);
+	uriFreeUriMembersA(&transformedUri);
 	return equal;
 }
 
@@ -383,11 +388,14 @@ void FourSuite::relativize_test_cases() {
 
 
 
-int FourSuite::testParseUri(const char * uriText) {
+int FourSuite::testParseUri(const char * uriText, const char ** expectedErrorPos) {
 	UriParserStateA state;
 	UriUriA uri;
 	state.uri = &uri;
 	int res = uriParseUriA(&state, uriText);
+	if (expectedErrorPos != NULL) {
+		*expectedErrorPos = state.errorPos;
+	}
 	uriFreeUriMembersA(&uri);
 	return res;
 }
@@ -400,8 +408,15 @@ bool FourSuite::testGoodUri(const char * uriText) {
 
 
 
-bool FourSuite::testBadUri(const char * uriText) {
-	return (testParseUri(uriText) == URI_ERROR_SYNTAX);
+bool FourSuite::testBadUri(const char * uriText, int expectedErrorOffset) {
+	const char * errorPos = NULL;
+	const int ret = testParseUri(uriText, &errorPos);
+	return ((ret == URI_ERROR_SYNTAX)
+			&& (errorPos != NULL)
+			&& (
+				(expectedErrorOffset == -1)
+				|| (errorPos == (uriText + expectedErrorOffset))
+			));
 }
 
 
@@ -496,45 +511,46 @@ void FourSuite::good_URI_references() {
 
 
 void FourSuite::bad_URI_references() {
-	TEST_ASSERT(testBadUri("beepbeep\x07\x07"));
-	TEST_ASSERT(testBadUri("\n"));
-	TEST_ASSERT(testBadUri("::")); // not OK, per Roy Fielding on the W3C uri list on 2004-04-01
+	TEST_ASSERT(testBadUri("beepbeep\x07\x07", 8));
+	TEST_ASSERT(testBadUri("\n", 0));
+	TEST_ASSERT(testBadUri("::", 0)); // not OK, per Roy Fielding on the W3C uri list on 2004-04-01
 
 	// the following test cases are from a Perl script by David A. Wheeler
 	// at http://www.dwheeler.com/secure-programs/url.pl
-	TEST_ASSERT(testBadUri("http://www yahoo.com"));
-	TEST_ASSERT(testBadUri("http://www.yahoo.com/hello world/"));
-	TEST_ASSERT(testBadUri("http://www.yahoo.com/yelp.html#\""));
+	TEST_ASSERT(testBadUri("http://www yahoo.com", 10));
+	TEST_ASSERT(testBadUri("http://www.yahoo.com/hello world/", 26));
+	TEST_ASSERT(testBadUri("http://www.yahoo.com/yelp.html#\"", 31));
 
 	// the following test cases are from a Haskell program by Graham Klyne
 	// at http://www.ninebynine.org/Software/HaskellUtils/Network/URITest.hs
-	TEST_ASSERT(testBadUri("[2010:836B:4179::836B:4179]"));
-	TEST_ASSERT(testBadUri(" "));
-	TEST_ASSERT(testBadUri("%"));
-	TEST_ASSERT(testBadUri("A%Z"));
-	TEST_ASSERT(testBadUri("%ZZ"));
-	TEST_ASSERT(testBadUri("%AZ"));
-	TEST_ASSERT(testBadUri("A C"));
-	TEST_ASSERT(testBadUri("A\\'C")); // r"A\'C"
-	TEST_ASSERT(testBadUri("A`C"));
-	TEST_ASSERT(testBadUri("A<C"));
-	TEST_ASSERT(testBadUri("A>C"));
-	TEST_ASSERT(testBadUri("A^C"));
-	TEST_ASSERT(testBadUri("A\\\\C")); // r'A\\C'
-	TEST_ASSERT(testBadUri("A{C"));
-	TEST_ASSERT(testBadUri("A|C"));
-	TEST_ASSERT(testBadUri("A}C"));
-	TEST_ASSERT(testBadUri("A[C"));
-	TEST_ASSERT(testBadUri("A]C"));
-	TEST_ASSERT(testBadUri("A[**]C"));
-	TEST_ASSERT(testBadUri("http://[xyz]/"));
-	TEST_ASSERT(testBadUri("http://]/"));
-	TEST_ASSERT(testBadUri("http://example.org/[2010:836B:4179::836B:4179]"));
-	TEST_ASSERT(testBadUri("http://example.org/abc#[2010:836B:4179::836B:4179]"));
-	TEST_ASSERT(testBadUri("http://example.org/xxx/[qwerty]#a[b]"));
+	TEST_ASSERT(testBadUri("[2010:836B:4179::836B:4179]", 0));
+	TEST_ASSERT(testBadUri(" ", 0));
+	TEST_ASSERT(testBadUri("%", 1));
+	TEST_ASSERT(testBadUri("A%Z", 2));
+	TEST_ASSERT(testBadUri("%ZZ", 1));
+	TEST_ASSERT(testBadUri("%AZ", 2));
+	TEST_ASSERT(testBadUri("A C", 1));
+	TEST_ASSERT(testBadUri("A\\'C", 1)); // r"A\'C"
+	TEST_ASSERT(testBadUri("A`C", 1));
+	TEST_ASSERT(testBadUri("A<C", 1));
+	TEST_ASSERT(testBadUri("A>C", 1));
+	TEST_ASSERT(testBadUri("A^C", 1));
+	TEST_ASSERT(testBadUri("A\\\\C", 1)); // r'A\\C'
+	TEST_ASSERT(testBadUri("A{C", 1));
+	TEST_ASSERT(testBadUri("A|C", 1));
+	TEST_ASSERT(testBadUri("A}C", 1));
+	TEST_ASSERT(testBadUri("A[C", 1));
+	TEST_ASSERT(testBadUri("A]C", 1));
+	TEST_ASSERT(testBadUri("A[**]C", 1));
+	TEST_ASSERT(testBadUri("http://[xyz]/", 8));
+	TEST_ASSERT(testBadUri("http://]/", 7));
+	TEST_ASSERT(testBadUri("http://example.org/[2010:836B:4179::836B:4179]", 19));
+	TEST_ASSERT(testBadUri("http://example.org/abc#[2010:836B:4179::836B:4179]", 23));
+	TEST_ASSERT(testBadUri("http://example.org/xxx/[qwerty]#a[b]", 23));
 
 	// from a post to the W3C uri list on 2004-02-17
-	TEST_ASSERT(testBadUri("http://w3c.org:80path1/path2"));
+	// breaks at 22 instead of 17 because everything up to that point is a valid userinfo
+	TEST_ASSERT(testBadUri("http://w3c.org:80path1/path2", 22));
 }
 
 
